#ifndef MSP_GL_CAMERA_H_
#define MSP_GL_CAMERA_H_

#include "matrix.h"
#include "vector.h"

namespace Msp {
namespace GL {

class Camera
{
private:
	Geometry::Angle<float> fov;
	float aspect;
	// Some compilers have "near" and "far" keywords
	float clip_near;
	float clip_far;
	float frustum_x;
	float frustum_y;
	Vector3 position;
	Vector3 look_dir;
	Vector3 up_dir;
	Matrix view_matrix;
	Matrix object_matrix;
	Matrix proj_matrix;

public:
	Camera();

	void set_field_of_view(const Geometry::Angle<float> &);
	void set_aspect(float);
	void set_depth_clip(float, float);
	void set_frustum_axis(float, float);
	const Geometry::Angle<float> &get_field_of_view() const { return fov; }
	float get_aspect() const { return aspect; }
	float get_near_clip() const { return clip_near; }
	float get_far_clip() const { return clip_far; }

	void set_position(const Vector3 &);
	void set_look_direction(const Vector3 &);
	void look_at(const Vector3 &);
	void set_up_direction(const Vector3 &);
	const Vector3 &get_position() const { return position; }
	const Vector3 &get_look_direction() const { return look_dir; }
	const Vector3 &get_up_direction() const { return up_dir; }

	/** Deprecated alias for get_view_matrix. */
	const Matrix &get_matrix() const { return get_view_matrix(); }

	/** Returns the view matrix, used to transform coordinates from world space
	to eye space. */
	const Matrix &get_view_matrix() const { return view_matrix; }

	/** Returns the object matrix, used to transform coordinates from eye space
	to world space. */
	const Matrix &get_object_matrix() const { return object_matrix; }

	/** Returns the projection matrix. */
	const Matrix &get_projection_matrix() const { return proj_matrix; }

	Vector3 project(const Vector4 &) const;
	Vector3 project(const Vector3 &) const;
	Vector4 unproject(const Vector4 &) const;

	void apply() const;

private:
	void update_projection_matrix();
	void update_object_matrix();
};

} // namespace GL
} // namespcae Msp

#endif
