#ifndef MSP_GL_LIGHT_H_
#define MSP_GL_LIGHT_H_

#include <vector>
#include "color.h"
#include "vector.h"

namespace Msp {
namespace GL {

class Matrix;
class ProgramData;

class Light
{
private:
	enum ParameterMask
	{
		DIFFUSE = 1,
		SPECULAR = 2,
		POSITION = 4,
		SPOT_DIR = 8,
		SPOT_EXP = 16,
		SPOT_CUTOFF = 32,
		ATTENUATION = 64
	};

	Color diffuse;
	Color specular;
	Vector4 position;
	Vector3 spot_dir;
	float spot_exp;
	float spot_cutoff;
	float attenuation[3];

public:
	Light();
	~Light();

private:
	void update_parameter(int, int = -1) const;

public:
	void set_diffuse(const Color &c);
	void set_specular(const Color &c);
	const Color &get_diffuse() const { return diffuse; }
	const Color &get_specular() const { return specular; }

	void set_position(const Vector4 &);
	const Vector4 &get_position() const { return position; }
	void set_spot_direction(const Vector3 &);
	void set_spot_exponent(float);
	void set_spot_cutoff(float);
	const Vector3 &get_spot_direction() const { return spot_dir; }
	float get_spot_exponent() const { return spot_exp; }
	float get_spot_cutoff() const { return spot_cutoff; }
	void set_attenuation(float, float, float);
	const float *get_attenuation() const { return attenuation; }

	void update_shader_data(ProgramData &, const Matrix &, unsigned) const;
	void bind() const { return bind_to(0); }
	void bind_to(unsigned) const;

	static const Light *current(unsigned = 0);
	static void unbind() { return unbind_from(0); }
	static void unbind_from(unsigned);
};

} // namespace GL
} // namespace Msp

#endif
