#include <stdexcept>
#include "gl.h"
#include "misc.h"
#include "lightunit.h"

using namespace std;

namespace Msp {
namespace GL {

vector<LightUnit> LightUnit::units;
LightUnit *LightUnit::cur_unit = 0;

LightUnit::LightUnit():
	light(0)
{ }

bool LightUnit::set_light(const Light *l)
{
	bool result = (l!=light);
	light = l;
	return result;
}

unsigned LightUnit::get_n_units()
{
	static int count = get_i(GL_MAX_LIGHTS);
	return count;
}

LightUnit &LightUnit::get_unit(unsigned n)
{
	if(n>=get_n_units())
		throw out_of_range("LightUnit::get_unit");

	if(units.size()<=n)
	{
		unsigned i = units.size();
		units.resize(n+1, LightUnit());
		for(; i<units.size(); ++i)
			units[i].index = i;
	}

	return units[n];
}

LightUnit *LightUnit::find_unit(const Light *l)
{
	for(vector<LightUnit>::iterator i=units.begin(); i!=units.end(); ++i)
		if(i->light==l)
			return &*i;
	return 0;
}

} // namespace GL
} // namespace Msp
