#ifndef MSP_GL_MATERIAL_H_
#define MSP_GL_MATERIAL_H_

#include <msp/datafile/objectloader.h>
#include "bindable.h"
#include "color.h"
#include "programdata.h"

namespace Msp {
namespace GL {

/**
Stores OpenGL material properties.  Since OpenGL does not support material
objects, application of material is done with several calls to glMaterial.
*/
class Material: public BindableWithDefault<Material>
{
public:
	class Loader: public DataFile::CollectionObjectLoader<Material>
	{
	private:
		bool srgb;

	public:
		Loader(Material &);
		Loader(Material &, Collection &);
	private:
		void init();

		Color make_color(float, float, float, float);
		void ambient(float, float, float, float);
		void diffuse(float, float, float, float);
		void specular(float, float, float, float);
		void emission(float, float, float, float);
		void shininess(float);
		void reflectivity(float);
	};

private:
	enum ParameterMask
	{
		AMBIENT = 1,
		DIFFUSE = 2,
		SPECULAR = 4,
		EMISSION = 8,
		SHININESS = 16
	};

	Color ambient;
	Color diffuse;
	Color specular;
	Color emission;
	float shininess;
	float reflectivity;
	ProgramData shdata;

public:
	Material();

private:
	void update_parameter(int) const;

public:
	void set_ambient(const Color &a);
	void set_diffuse(const Color &d);
	void set_specular(const Color &s);
	void set_emission(const Color &e);
	void set_shininess(float s);
	void set_reflectivity(float);
	const Color &get_ambient() const { return ambient; }
	const Color &get_diffuse() const { return diffuse; }
	const Color &get_specular() const { return specular; }
	const Color &get_emission() const { return emission; }
	float get_shininess() const { return shininess; }
	float get_reflectivity() const { return reflectivity; }
	const ProgramData &get_shader_data() const { return shdata; }
	void bind() const;
};

} // namespace GL
} // namespace Msp

#endif
