#include <algorithm>
#include "orderedscene.h"
#include "renderer.h"

namespace Msp {
namespace GL {

void OrderedScene::add(const Renderable &r)
{
	renderables.push_back(&r);
}

void OrderedScene::remove(const Renderable &r)
{
	RenderableList::iterator end = std::remove(renderables.begin(), renderables.end(), &r);
	renderables.erase(end, renderables.end());
}

void OrderedScene::prepend(const Renderable &r)
{
	renderables.push_front(&r);
}

void OrderedScene::insert(unsigned index, const Renderable &r)
{
	RenderableList::iterator i = renderables.begin();
	for(; (i!=renderables.end() && index); ++i, --index) ;
	renderables.insert(i, &r);
}

void OrderedScene::insert_after(const Renderable &after, const Renderable &r)
{
	RenderableList::iterator i = renderables.begin();
	for(; (i!=renderables.end() && *i!=&after); ++i) ;
	renderables.insert(i, &r);
}

void OrderedScene::setup_frame() const
{
	for(RenderableList::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		(*i)->setup_frame();
}

void OrderedScene::finish_frame() const
{
	for(RenderableList::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		(*i)->finish_frame();
}

void OrderedScene::render(Renderer &renderer, const Tag &tag) const
{
	setup_cullers(renderer);
	for(RenderableList::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		if(!cull(renderer, **i))
			renderer.render(**i, tag);
}

} // namespace GL
} // namespace Msp
