#include <msp/gl/extensions/arb_texture_float.h>
#include <msp/gl/extensions/ext_bgra.h>
#include <msp/strings/format.h>
#include "pixelformat.h"

using namespace std;

namespace Msp {
namespace GL {

void operator>>(const LexicalConverter &conv, PixelFormat &fmt)
{
	if(conv.get()=="STENCIL_INDEX")
		fmt = STENCIL_INDEX;
	else if(conv.get()=="DEPTH_COMPONENT")
		fmt = DEPTH_COMPONENT;
	else if(conv.get()=="RGB")
		fmt = RGB;
	else if(conv.get()=="RGBA")
		fmt = RGBA;
	else if(conv.get()=="BGR")
		fmt = BGR;
	else if(conv.get()=="BGRA")
		fmt = BGRA;
	else if(conv.get()=="SRGB")
		fmt = SRGB;
	else if(conv.get()=="SRGB_ALPHA")
		fmt = SRGB_ALPHA;
	else if(conv.get()=="LUMINANCE")
		fmt = LUMINANCE;
	else if(conv.get()=="LUMINANCE_ALPHA")
		fmt = LUMINANCE_ALPHA;
	else if(conv.get()=="SLUMINANCE")
		fmt = SLUMINANCE;
	else if(conv.get()=="SLUMINANCE_ALPHA")
		fmt = SLUMINANCE_ALPHA;
	else
		throw lexical_error(format("conversion of '%s' to PixelFormat", conv.get()));
}

PixelFormat pixelformat_from_graphics(Graphics::PixelFormat pf)
{
	switch(pf)
	{
	case Graphics::LUMINANCE: return LUMINANCE;
	case Graphics::LUMINANCE_ALPHA: return LUMINANCE_ALPHA;
	case Graphics::RGB: return RGB;
	case Graphics::RGBX:
	case Graphics::RGBA: return RGBA;
	case Graphics::BGR: return BGR;
	case Graphics::BGRX:
	case Graphics::BGRA: return BGRA;
	default: throw invalid_argument("pixelformat_from_graphics");
	}
}

PixelFormat storage_pixelformat_from_graphics(Graphics::PixelFormat pf)
{
	switch(pf)
	{
	case Graphics::RGBX:
	case Graphics::BGR:
	case Graphics::BGRX: return RGB;
	case Graphics::BGRA: return RGBA;
	default: return pixelformat_from_graphics(pf);
	}
}

PixelFormat get_base_pixelformat(PixelFormat pf)
{
	switch(pf)
	{
	case RGB8:
	case RGB16F:
	case RGB32F:
	case SRGB:
	case SRGB8: return RGB;
	case RGBA8:
	case RGBA16F:
	case RGBA32F:
	case SRGB_ALPHA:
	case SRGB8_ALPHA8: return RGBA;
	case LUMINANCE8:
	case LUMINANCE16F:
	case LUMINANCE32F:
	case SLUMINANCE:
	case SLUMINANCE8: return LUMINANCE;
	case LUMINANCE_ALPHA8:
	case LUMINANCE_ALPHA16F:
	case LUMINANCE_ALPHA32F:
	case SLUMINANCE_ALPHA:
	case SLUMINANCE8_ALPHA8: return LUMINANCE_ALPHA;
	default: return pf;
	}
}

PixelFormat get_srgb_pixelformat(PixelFormat pf)
{
	switch(pf)
	{
	case RGB: return SRGB;
	case RGBA: return SRGB_ALPHA;
	case RGB8: return SRGB8;
	case RGBA8: return SRGB8_ALPHA8;
	case LUMINANCE: return SLUMINANCE;
	case LUMINANCE8: return SLUMINANCE8;
	case LUMINANCE_ALPHA: return SLUMINANCE_ALPHA;
	case LUMINANCE_ALPHA8: return SLUMINANCE8_ALPHA8;
	default: return pf;
	}
}

unsigned get_component_count(PixelFormat pf)
{
	switch(get_base_pixelformat(pf))
	{
	case STENCIL_INDEX:
	case DEPTH_COMPONENT:
	case LUMINANCE:
	case SLUMINANCE:
		return 1;
	case LUMINANCE_ALPHA:
	case SLUMINANCE_ALPHA:
		return 2;
	case RGB:
	case BGR:
		return 3;
	case RGBA:
	case BGRA:
		return 4;
	default:
		throw invalid_argument("get_component_count");
	}
}

unsigned get_component_size(PixelFormat pf)
{
	switch(pf)
	{
	case RGB16F:
	case RGBA16F:
	case LUMINANCE16F:
	case LUMINANCE_ALPHA16F:
		return 2;
	case RGB32F:
	case RGBA32F:
	case LUMINANCE32F:
	case LUMINANCE_ALPHA32F:
		return 4;
	default:
		return 1;
	}
}

unsigned get_pixel_size(PixelFormat pf)
{
	return get_component_count(pf)*get_component_size(pf);
}

void require_pixelformat(PixelFormat pf)
{
	switch(pf)
	{
	case RGB16F:
	case RGB32F:
	case RGBA16F:
	case RGBA32F:
	case LUMINANCE16F:
	case LUMINANCE32F:
	case LUMINANCE_ALPHA16F:
	case LUMINANCE_ALPHA32F:
		{ static Require _req(ARB_texture_float); }
		break;
	case SRGB:
	case SRGB8:
	case SRGB_ALPHA:
	case SRGB8_ALPHA8:
	case SLUMINANCE:
	case SLUMINANCE8:
	case SLUMINANCE_ALPHA:
	case SLUMINANCE8_ALPHA8:
		{ static Require _req(EXT_texture_sRGB); }
		break;
	case BGR:
	case BGRA:
		{ static Require _req(EXT_bgra); }
		break;
	default:
		break;
	}
}

} // namespace GL
} // namespace Msp
