#include <msp/gl/extensions/ext_framebuffer_multisample.h>
#include <msp/gl/extensions/ext_framebuffer_object.h>
#include "renderbuffer.h"

namespace Msp {
namespace GL {

Renderbuffer::Renderbuffer()
{
	static Require _req(EXT_framebuffer_object);

	glGenRenderbuffers(1, &id);
}

Renderbuffer::~Renderbuffer()
{
	glDeleteRenderbuffers(1, &id);
}

void Renderbuffer::storage(PixelFormat fmt, unsigned wd, unsigned ht)
{
	require_pixelformat(fmt);
	BindRestore _bind(this);
	width = wd;
	height = ht;
	glRenderbufferStorage(GL_RENDERBUFFER, fmt, width, height);
}

void Renderbuffer::storage_multisample(unsigned samples, PixelFormat fmt, unsigned wd, unsigned ht)
{
	static Require _req(EXT_framebuffer_multisample);
	require_pixelformat(fmt);

	BindRestore _bind(this);
	width = wd;
	height = ht;
	glRenderbufferStorageMultisample(GL_RENDERBUFFER, samples, fmt, width, height);
}

void Renderbuffer::bind() const
{
	if(set_current(this))
		glBindRenderbuffer(GL_RENDERBUFFER, id);
}

void Renderbuffer::unbind()
{
	if(set_current(0))
		glBindRenderbuffer(GL_RENDERBUFFER, 0);
}

} // namespace GL
} // namespace Msp
