#include <msp/gl/extensions/arb_fragment_shader.h>
#include <msp/gl/extensions/arb_shader_objects.h>
#include <msp/gl/extensions/arb_vertex_shader.h>
#include "error.h"
#include "misc.h"
#include "shader.h"

using namespace std;

namespace Msp {
namespace GL {

Shader::Shader(GLenum t)
{
	init(t);
}

Shader::Shader(GLenum t, const string &src)
{
	init(t);

	source(src);
	compile();
}

void Shader::init(GLenum t)
{
	compiled = false;

	if(t==GL_FRAGMENT_SHADER)
		static Require _req(ARB_fragment_shader);
	else if(t==GL_VERTEX_SHADER)
		static Require _req(ARB_vertex_shader);

	id = glCreateShader(t);
}

Shader::~Shader()
{
	glDeleteShader(id);
}

void Shader::source(unsigned count, const char **str, const int *len)
{
	glShaderSource(id, count, str, len);
}

void Shader::source(const string &str)
{
	source(str.data(), str.size());
}

void Shader::source(const char *str, int len)
{
	source(1, &str, &len);
}

void Shader::compile()
{
	glCompileShader(id);
	compiled = get_shader_i(id, GL_COMPILE_STATUS);
	if(!compiled)
		throw compile_error(get_info_log());
}

string Shader::get_info_log() const
{
	GLsizei len = get_shader_i(id, GL_INFO_LOG_LENGTH);
	char *buf = new char[len+1];
	glGetShaderInfoLog(id, len+1, &len, buf);
	string log(buf, len);
	delete[] buf;
	return log;
}


VertexShader::VertexShader():
	Shader(GL_VERTEX_SHADER)
{ }

VertexShader::VertexShader(const string &src):
	Shader(GL_VERTEX_SHADER, src)
{ }


FragmentShader::FragmentShader():
	Shader(GL_FRAGMENT_SHADER)
{ }

FragmentShader::FragmentShader(const string &src):
	Shader(GL_FRAGMENT_SHADER, src)
{ }

} // namespace GL
} // namespace Msp
