#ifndef MSP_GL_VERTEXARRAY_H_
#define MSP_GL_VERTEXARRAY_H_

#include <climits>
#include <vector>
#include <msp/core/refptr.h>
#include <msp/datafile/loader.h>
#include "bindable.h"
#include "bufferable.h"
#include "datatype.h"
#include "primitivetype.h"
#include "vertexarraybuilder.h"
#include "vertexformat.h"

namespace Msp {
namespace GL {

class Buffer;

class VertexArray: public Bindable<VertexArray>, public Bufferable
{
public:
	class Loader: public DataFile::Loader, public VertexArrayBuilder
	{
	public:
		Loader(VertexArray &);
	};

private:
	struct Array
	{
		unsigned char component;
		unsigned char offset;

		Array();
	};

	VertexFormat format;
	std::vector<float> data;
	unsigned stride;
	std::vector<Array> arrays;

	VertexArray(const VertexArray &);
	VertexArray &operator=(const VertexArray &);
public:
	VertexArray(const VertexFormat &);
	~VertexArray();

	void reset(const VertexFormat &);
	const VertexFormat &get_format() const { return format; }
private:
	static unsigned get_array_slot(unsigned char);

public:
	/// Deprecated alias for use_buffer
	void use_vertex_buffer(Buffer *b) { use_buffer(b); }

	void clear();
	void reserve(unsigned);
	float *append();
	float *modify(unsigned);
private:
	virtual unsigned get_data_size() const;
	virtual const void *get_data_pointer() const { return &data[0]; }

public:
	unsigned size() const { return data.size()/stride; }
	const std::vector<float> &get_data() const { return data; }
	const float *operator[](unsigned i) const { return &data[0]+i*stride; }

	void bind() const;
	void apply() const { bind(); }
private:
	static void apply_arrays(const std::vector<Array> *, const std::vector<Array> *, const float *, unsigned);
public:
	static void unbind();
};

} // namespace GL
} // namespace Msp

#endif
