#include "camera.h"
#include "renderer.h"
#include "zsortedscene.h"

using namespace std;

namespace Msp {
namespace GL {

ZSortedScene::ZSortedScene():
	order(BACK_TO_FRONT),
	reference(FURTHEST)
{ }

void ZSortedScene::set_order(SortOrder o)
{
	order = o;
}

void ZSortedScene::set_reference(DepthReference r)
{
	reference = r;
}

void ZSortedScene::render(Renderer &renderer, const Tag &tag) const
{
	const Camera *camera = renderer.get_camera();
	if(!camera)
		return SimpleScene::render(renderer, tag);

	const Matrix &view_matrix = camera->get_matrix();

	std::vector<DepthRenderable> sorted_renderables;
	sorted_renderables.reserve(renderables.size());

	float radius_factor = 1.0f-reference;
	float sign = order*2.0f-1.0f;

	setup_cullers(renderer);
	for(RenderableSet::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		if(!cull(renderer, **i))
		{
			float z = 0;
			if(const Matrix *model_matrix = (*i)->get_matrix())
			{
				if(const Geometry::BoundingSphere<float, 3> *bsphere = (*i)->get_bounding_sphere())
					z = (view_matrix**model_matrix*Vector4(bsphere->get_center(), 1.0f)).z+bsphere->get_radius()*radius_factor;
				else
					z = (view_matrix**model_matrix*Vector4(0.0f, 0.0f, 0.0f, 1.0f)).z;
			}
			sorted_renderables.push_back(DepthRenderable(z*sign, *i));
		}

	sort(sorted_renderables.begin(), sorted_renderables.end());

	for(vector<DepthRenderable>::const_iterator i=sorted_renderables.begin(); i!=sorted_renderables.end(); ++i)
		i->renderable->render(renderer, tag);
}


ZSortedScene::DepthRenderable::DepthRenderable(float d, const Renderable *r):
	depth(d),
	renderable(r)
{ }

} // namespace GL
} // namespace Msp
